@@ -15,6 +15,7 @@
  */
 
 #include "kerncompat.h"
+#include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -71,7 +72,7 @@ static int parse_one_token(const char *arg, const struct cmd_group *grp,
 
 		rest = skip_prefix(arg, cmd->token);
 		if (!rest) {
-			if (!prefixcmp(cmd->token, arg)) {
+			if (!string_has_prefix(cmd->token, arg)) {
 				if (abbrev_cmd) {
 					/*
 					 * If this is abbreviated, it is
@@ -420,6 +421,8 @@ int main(int argc, char **argv)
 		return convert_main(argc, argv);
 	} else if (!strcmp(bname, "btrfstune")) {
 		return btrfstune_main(argc, argv);
+	} else if (!strcmp(bname, "btrfs-find-root")) {
+		return find_root_main(argc, argv);
 #endif
 	} else {
 		int shift;
