@@ -25,9 +25,8 @@
 #include <getopt.h>
 #include <dirent.h>
 #include <stdbool.h>
-#include "kernel-shared/uapi/btrfs.h"
-#include "kernel-shared/ctree.h"
 #include "kernel-shared/zoned.h"
+#include "kernel-shared/volumes.h"
 #include "common/string-table.h"
 #include "common/utils.h"
 #include "common/help.h"
