@@ -17,11 +17,15 @@
 #include "kerncompat.h"
 #include <errno.h>
 #include <stdio.h>
+#include "kernel-shared/accessors.h"
+#include "kernel-shared/uapi/btrfs.h"
+#include "kernel-shared/uapi/btrfs_tree.h"
 #include "kernel-shared/ctree.h"
 #include "kernel-shared/disk-io.h"
 #include "kernel-shared/transaction.h"
 #include "common/messages.h"
 #include "common/extent-cache.h"
+#include "tune/tune.h"
 
 /* After this many block groups we need to commit transaction. */
 #define BLOCK_GROUP_BATCH	64
@@ -236,7 +240,7 @@ iterate_bgs:
 	 * Remove block group tree, at this stage, the block group tree root
 	 * should be empty.
 	 */
-	ASSERT(btrfs_header_nritems(fs_info->block_group_root->node) == 0);
+	UASSERT(btrfs_header_nritems(fs_info->block_group_root->node) == 0);
 	ret = btrfs_delete_and_free_root(trans, fs_info->block_group_root);
 	if (ret < 0) {
 		errno = -ret;
